using System;
using System.Collections;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;

class MainFrame : Form
{
	SqlDocument _SqlDocument;
	ToolBar _ToolBar;
	SqlBox _SqlBox;
	Splitter _Splitter;
	Panel _Panel1;
	OutputWindow[] _Output;
	SheetView _SheetView;
	StatusBar _StatusBar;
	StatusBarPanel _StatusBarPanel1;
	MenuItem _MenuItemEditUndo;
	MenuItem _MenuItemEditCut;
	MenuItem _MenuItemEditCopy;
	MenuItem _MenuItemEditPaste;
	MenuItem _MenuItemEditDelete;
	MenuItem _MenuItemEditSelectAll;

	MainFrame()
	{
		Width = 640;
		Height = 480;

		CreateMenu();

		_ToolBar = new ToolBar();


		_SqlBox = new SqlBox();
		_SqlBox.Dock = DockStyle.Top;
		_SqlBox.Height = this.Height / 3 * 2;
		_SqlBox.WordWrap = false;
		_SqlBox.ContextMenu = CreateContextMenu(_SqlBox);
		_SqlBox.Font = new Font("Courier New", 10, GraphicsUnit.Point);

		_Splitter = new Splitter();
		_Splitter.Dock = DockStyle.Top;
		_Splitter.MinSize = 0;
		_Splitter.MinExtra = 0;

		_Panel1 = new Panel();
		_Panel1.Dock = DockStyle.Fill;

		_Output = new OutputWindow[2];
		_Output[0] = new OutputWindow();
		_Output[0].Dock = DockStyle.Fill;
		_Output[0].Multiline = true;
		_Output[0].WordWrap = false;
		_Output[0].ScrollBars = RichTextBoxScrollBars.Both;
		_Output[0].ReadOnly = true;
		_Output[0].ContextMenu = CreateContextMenu(_Output[0]);
		_Output[0].Font = _SqlBox.Font;

		_Output[1] = new OutputWindow();
		_Output[1].Dock = DockStyle.Fill;
		_Output[1].Multiline = true;
		_Output[1].WordWrap = false;
		_Output[1].ScrollBars = RichTextBoxScrollBars.Both;
		_Output[1].ReadOnly = true;
		_Output[1].Visible = false;
		_Output[1].ContextMenu = CreateContextMenu(_Output[1]);
		_Output[1].Text = "false";
		_Output[1].Font = _Output[0].Font;

		_SheetView = new SheetView();
		_SheetView.Dock = DockStyle.Bottom;
		_SheetView.Height = 18;
		_SheetView.Font = new Font("verdana", 8, GraphicsUnit.Point);
		_SheetView.Sheets.Add(new Sheet("Output 1"));
		_SheetView.Sheets.Add(new Sheet("Output 2"));
		_SheetView.SelectionChanged += new EventHandler(OnOutputWindowChanged);
		_SheetView.SelectedIndex = 1;

		_StatusBar = new StatusBar();
		_StatusBarPanel1 = new StatusBarPanel();
		_StatusBarPanel1.BorderStyle = StatusBarPanelBorderStyle.Sunken;
		_StatusBarPanel1.AutoSize = StatusBarPanelAutoSize.Spring;
		_StatusBar.Panels.Add(_StatusBarPanel1);
		_StatusBar.ShowPanels = true;

		_Panel1.Controls.AddRange(
			new Control[]
			{
				_Output[0],
				_Output[1],
				_SheetView,
				_StatusBar
			}
		);

		Controls.AddRange(
			new Control[]
			{
				_Panel1,
				_Splitter,
				_SqlBox,
				_ToolBar
			}
		);

		_SqlDocument = new SqlDocument();
		_SqlDocument.Control = _SqlBox;

		UpdateTitle();
	}

	void UpdateTitle()
	{
		this.Text = _SqlDocument.Title + " - " + "SQLPad";
	}

	protected override void OnLoad(EventArgs e)
	{
		this.ActiveControl = _SqlBox;
	}

	void CreateMenu()
	{
		MenuItem menuItem;

		// file
		MenuItem fileMenu = new MenuItem("&File");
		fileMenu.Popup += new EventHandler(OnFilePopup);

		menuItem = new MenuItem("&New");
		menuItem.Shortcut = Shortcut.CtrlN;
		menuItem.Click += new EventHandler(OnFileNew);
		fileMenu.MenuItems.Add(menuItem);

		menuItem = new MenuItem("&Open...");
		menuItem.Shortcut = Shortcut.CtrlO;
		menuItem.Click += new EventHandler(OnFileOpen);
		fileMenu.MenuItems.Add(menuItem);

		menuItem = new MenuItem("&Save");
		menuItem.Shortcut = Shortcut.CtrlS;
		menuItem.Click += new EventHandler(OnFileSave);
		fileMenu.MenuItems.Add(menuItem);

		menuItem = new MenuItem("Save &As...");
		menuItem.Shortcut = Shortcut.CtrlShiftS;
		menuItem.Click += new EventHandler(OnFileSaveAs);
		fileMenu.MenuItems.Add(menuItem);

		menuItem = new MenuItem("-");
		fileMenu.MenuItems.Add(menuItem);

		menuItem = new MenuItem("E&xit");
		menuItem.Shortcut = Shortcut.AltF4;
		menuItem.Click += new EventHandler(OnFileExit);
		fileMenu.MenuItems.Add(menuItem);

		// edit
		MenuItem editMenu = new MenuItem("&Edit");
		editMenu.Popup += new EventHandler(OnEditPopup);

		_MenuItemEditUndo = new MenuItem("&Undo");
		_MenuItemEditUndo.Shortcut = Shortcut.CtrlZ;
		_MenuItemEditUndo.Click += new EventHandler(OnEditUndo);
		editMenu.MenuItems.Add(_MenuItemEditUndo);

		menuItem = new MenuItem("-");
		editMenu.MenuItems.Add(menuItem);

		_MenuItemEditCut = new MenuItem("Cu&t");
		_MenuItemEditCut.Shortcut = Shortcut.CtrlX;
		_MenuItemEditCut.Click += new EventHandler(OnEditCut);
		editMenu.MenuItems.Add(_MenuItemEditCut);

		_MenuItemEditCopy = new MenuItem("&Copy");
		_MenuItemEditCopy.Shortcut = Shortcut.CtrlC;
		_MenuItemEditCopy.Click += new EventHandler(OnEditCopy);
		editMenu.MenuItems.Add(_MenuItemEditCopy);

		_MenuItemEditPaste = new MenuItem("&Paste");
		_MenuItemEditPaste.Shortcut = Shortcut.CtrlV;
		_MenuItemEditPaste.Click += new EventHandler(OnEditPaste);
		editMenu.MenuItems.Add(_MenuItemEditPaste);

		_MenuItemEditDelete = new MenuItem("&Delete");
		_MenuItemEditDelete.Shortcut = Shortcut.Del;
		_MenuItemEditDelete.Click += new EventHandler(OnEditDelete);
		editMenu.MenuItems.Add(_MenuItemEditDelete);

		menuItem = new MenuItem("-");
		editMenu.MenuItems.Add(menuItem);

		_MenuItemEditSelectAll = new MenuItem("Select &All");
		_MenuItemEditSelectAll.Shortcut = Shortcut.CtrlA;
		_MenuItemEditSelectAll.Click += new EventHandler(OnEditSelectAll);
		editMenu.MenuItems.Add(_MenuItemEditSelectAll);

		// view
		MenuItem viewMenu = new MenuItem("&View");

		// help
		MenuItem helpMenu = new MenuItem("&Help");

		this.Menu = new MainMenu();
		this.Menu.MenuItems.Add(fileMenu);
		this.Menu.MenuItems.Add(editMenu);
		this.Menu.MenuItems.Add(viewMenu);
		this.Menu.MenuItems.Add(helpMenu);
	}

	SqlContextMenu CreateContextMenu(Control owner)
	{
		MenuItem item;

		SqlContextMenu contextMenu = new SqlContextMenu();
		contextMenu.Owner = owner;
		contextMenu.Popup += new EventHandler(OnContextMenuPopup);

		item = new MenuItem("&Undo");
		item.Click += new EventHandler(OnEditUndo);
		contextMenu.MenuItems.Add(item);
		contextMenu.MenuItems.Add("-");
		item = new MenuItem("Cu&t");
		item.Click += new EventHandler(OnEditCut);
		contextMenu.MenuItems.Add(item);
		item = new MenuItem("&Copy");
		item.Click += new EventHandler(OnEditCopy);
		contextMenu.MenuItems.Add(item);
		item = new MenuItem("&Paste");
		item.Click += new EventHandler(OnEditPaste);
		contextMenu.MenuItems.Add(item);
		item = new MenuItem("&Delete");
		item.Click += new EventHandler(OnEditDelete);
		contextMenu.MenuItems.Add(item);
		contextMenu.MenuItems.Add("-");
		item = new MenuItem("Select &All");
		item.Click += new EventHandler(OnEditSelectAll);
		contextMenu.MenuItems.Add(item);

		return contextMenu;
	}

	void OnContextMenuPopup(Object sender, EventArgs e)
	{
		// must be in same order as in CreateContextMenu()

		SqlContextMenu contextMenu = sender as SqlContextMenu;

		if (contextMenu.Owner == _SqlBox)
		{
			contextMenu.MenuItems[0].Enabled = _SqlBox.CanUndo;
			contextMenu.MenuItems[2].Enabled = _SqlBox.GetSelectedTextLength() > 0;
			contextMenu.MenuItems[3].Enabled = _SqlBox.GetSelectedTextLength() > 0;
			contextMenu.MenuItems[4].Enabled = Clipboard.GetDataObject().
											   GetDataPresent(DataFormats.Text);
			contextMenu.MenuItems[5].Enabled = _SqlBox.GetSelectedTextLength() > 0;
			contextMenu.MenuItems[7].Enabled = _SqlBox.GetTextLength() > 0;
		}
		else if (contextMenu.Owner == _Output[_SheetView.SelectedIndex])
		{
			contextMenu.MenuItems[0].Enabled = false;
			contextMenu.MenuItems[2].Enabled = false;
			contextMenu.MenuItems[3].Enabled =
				_Output[_SheetView.SelectedIndex].GetSelectedTextLength() > 0;
			contextMenu.MenuItems[4].Enabled = false;
			contextMenu.MenuItems[5].Enabled = false;
			contextMenu.MenuItems[7].Enabled =
				_Output[_SheetView.SelectedIndex].GetTextLength() > 0;
		}
	}

	void OnOutputWindowChanged(Object sender, EventArgs e)
	{
		_Output[0].Visible = false;
		_Output[1].Visible = false;
		_Output[_SheetView.SelectedIndex].Visible = true;
	}

	void OnFilePopup(Object sender, EventArgs e)
	{
	}

	void OnFileNew(Object sender, EventArgs e)
	{
		if (_SqlDocument.New())
		{
			UpdateTitle();
		}
	}

	void OnFileOpen(Object sender, EventArgs e)
	{
		if (_SqlDocument.Open())
		{
			UpdateTitle();
		}
	}

	void OnFileSave(Object sender, EventArgs e)
	{
		if (_SqlDocument.Save())
		{
			UpdateTitle();
		}
	}

	void OnFileSaveAs(Object sender, EventArgs e)
	{
		if (_SqlDocument.SaveAs())
		{
			UpdateTitle();
		}
	}

	void OnFileExit(Object sender, EventArgs e)
	{
		Close();
		Application.Exit();
	}

	void OnEditPopup(Object sender, EventArgs e)
	{
		_MenuItemEditUndo.Enabled = _SqlBox.Focused &&
									_SqlBox.CanUndo;
		_MenuItemEditCut.Enabled = _SqlBox.Focused &&
								   _SqlBox.GetSelectedTextLength() > 0;
		_MenuItemEditCopy.Enabled = (_SqlBox.Focused &&
									_SqlBox.GetSelectedTextLength() > 0) ||
									(_Output[0].Focused &&
									_Output[0].GetSelectedTextLength() > 0) ||
									(_Output[1].Focused &&
									_Output[1].GetSelectedTextLength() > 0);
		_MenuItemEditPaste.Enabled = _SqlBox.Focused &&
									 Clipboard.GetDataObject().
									 GetDataPresent(DataFormats.Text);
		_MenuItemEditDelete.Enabled = _SqlBox.Focused &&
									  _SqlBox.GetSelectedTextLength() > 0;
		_MenuItemEditSelectAll.Enabled = (_SqlBox.Focused &&
										 _SqlBox.GetTextLength() > 0) ||
										 (_Output[0].Focused &&
										 _Output[0].GetTextLength() > 0) ||
										 (_Output[1].Focused &&
										 _Output[1].GetTextLength() > 0);
	}

	void OnEditUndo(Object sender, EventArgs e)
	{
		_SqlBox.Undo();
		//_SqlBox.ClearUndo();
	}

	void OnEditCut(Object sender, EventArgs e)
	{
		_SqlBox.Cut();
	}

	void OnEditCopy(Object sender, EventArgs e)
	{
		if (_SqlBox.Focused)
			_SqlBox.Copy();
		else if (_Output[_SheetView.SelectedIndex].Focused)
			_Output[_SheetView.SelectedIndex].Copy();
	}

	void OnEditPaste(Object sender, EventArgs e)
	{
		_SqlBox.Paste();
	}

	void OnEditDelete(Object sender, EventArgs e)
	{
		_SqlBox.SelectedText = "";
	}

	void OnEditSelectAll(Object sender, EventArgs e)
	{
		if (_SqlBox.Focused)
			_SqlBox.SelectAllText();
		else if (_Output[_SheetView.SelectedIndex].Focused)
			_Output[_SheetView.SelectedIndex].SelectAllText();
	}

	[STAThread]	// make clipboard GetDataObject() work
	static void Main()
	{
		System.Windows.Forms.Application.Run(new MainFrame());
	}
}



